@@ -20,8 +20,7 @@
  */
 
 #include "config.h"
-#include "pango-fontmap-private.h"
-#include "pango-fontset-private.h"
+#include "pango-fontmap.h"
 #include "pango-impl-utils.h"
 #include <stdlib.h>
 
@@ -83,8 +82,8 @@ pango_font_map_create_context (PangoFontMap *fontmap)
  *
  * Load the font in the fontmap that is the closest match for @desc.
  *
- * Returns: (transfer full) (nullable): the newly allocated #PangoFont
- *          loaded, or %NULL if no font matched.
+ * Returns: (transfer full): the newly allocated #PangoFont loaded,
+ *          or %NULL if no font matched.
  **/
 PangoFont *
 pango_font_map_load_font  (PangoFontMap               *fontmap,
@@ -125,8 +124,8 @@ pango_font_map_list_families (PangoFontMap      *fontmap,
  * Load a set of fonts in the fontmap that can be used to render
  * a font matching @desc.
  *
- * Returns: (transfer full) (nullable): the newly allocated
- *          #PangoFontset loaded, or %NULL if no font matched.
+ * Returns: (transfer full): the newly allocated #PangoFontset
+ *          loaded, or %NULL if no font matched.
  **/
 PangoFontset *
 pango_font_map_load_fontset (PangoFontMap                 *fontmap,
@@ -146,12 +145,32 @@ pango_font_map_fontset_add_fonts (PangoFontMap          *fontmap,
 				  PangoFontDescription  *desc,
 				  const char            *family)
 {
+  char **aliases;
+  int n_aliases;
+  int j;
   PangoFont *font;
 
-  pango_font_description_set_family_static (desc, family);
-  font = pango_font_map_load_font (fontmap, context, desc);
-  if (font)
-    pango_fontset_simple_append (fonts, font);
+  pango_lookup_aliases (family,
+			&aliases,
+			&n_aliases);
+
+  if (n_aliases)
+    {
+      for (j = 0; j < n_aliases; j++)
+	{
+	  pango_font_description_set_family_static (desc, aliases[j]);
+	  font = pango_font_map_load_font (fontmap, context, desc);
+	  if (font)
+	    pango_fontset_simple_append (fonts, font);
+	}
+    }
+  else
+    {
+      pango_font_description_set_family_static (desc, family);
+      font = pango_font_map_load_font (fontmap, context, desc);
+      if (font)
+	pango_fontset_simple_append (fonts, font);
+    }
 }
 
 static PangoFontset *
@@ -165,8 +184,7 @@ pango_font_map_real_load_fontset (PangoFontMap               *fontmap,
   char **families;
   int i;
   PangoFontsetSimple *fonts;
-  static GHashTable *warned_fonts = NULL; /* MT-safe */
-  G_LOCK_DEFINE_STATIC (warned_fonts);
+  static GHashTable *warned_fonts = NULL;
 
   family = pango_font_description_get_family (desc);
   families = g_strsplit (family ? family : "", ",", -1);
@@ -195,7 +213,6 @@ pango_font_map_real_load_fontset (PangoFontMap               *fontmap,
       ctmp1 = pango_font_description_to_string (desc);
       pango_font_description_set_family_static (tmp_desc, "Sans");
 
-      G_LOCK (warned_fonts);
       if (!warned_fonts || !g_hash_table_lookup (warned_fonts, ctmp1))
 	{
 	  if (!warned_fonts)
@@ -208,7 +225,6 @@ pango_font_map_real_load_fontset (PangoFontMap               *fontmap,
 		     "expect ugly output.", ctmp1, ctmp2);
 	  g_free (ctmp2);
 	}
-      G_UNLOCK (warned_fonts);
       g_free (ctmp1);
 
       pango_font_map_fontset_add_fonts (fontmap,
@@ -231,7 +247,6 @@ pango_font_map_real_load_fontset (PangoFontMap               *fontmap,
       pango_font_description_set_variant (tmp_desc, PANGO_VARIANT_NORMAL);
       pango_font_description_set_stretch (tmp_desc, PANGO_STRETCH_NORMAL);
 
-      G_LOCK (warned_fonts);
       if (!warned_fonts || !g_hash_table_lookup (warned_fonts, ctmp1))
 	{
 	  g_hash_table_insert (warned_fonts, g_strdup (ctmp1), GINT_TO_POINTER (1));
@@ -242,7 +257,7 @@ pango_font_map_real_load_fontset (PangoFontMap               *fontmap,
 		     "expect ugly output.", ctmp1, ctmp2);
 	  g_free (ctmp2);
 	}
-      G_UNLOCK (warned_fonts);
+
       g_free (ctmp1);
 
       pango_font_map_fontset_add_fonts (fontmap,
@@ -276,7 +291,6 @@ pango_font_map_real_load_fontset (PangoFontMap               *fontmap,
  *  or freed.
  *
  * Since: 1.4
- * Deprecated: 1.38
  **/
 const char *
 pango_font_map_get_shape_engine_type (PangoFontMap *fontmap)
@@ -286,56 +300,3 @@ pango_font_map_get_shape_engine_type (PangoFontMap *fontmap)
   return PANGO_FONT_MAP_GET_CLASS (fontmap)->shape_engine_type;
 }
 
-/**
- * pango_font_map_get_serial:
- * @fontmap: a #PangoFontMap
- *
- * Returns the current serial number of @fontmap.  The serial number is
- * initialized to an small number larger than zero when a new fontmap
- * is created and is increased whenever the fontmap is changed. It may
- * wrap, but will never have the value 0. Since it can wrap, never compare
- * it with "less than", always use "not equals".
- *
- * The fontmap can only be changed using backend-specific API, like changing
- * fontmap resolution.
- *
- * This can be used to automatically detect changes to a #PangoFontMap, like
- * in #PangoContext.
- *
- * Return value: The current serial number of @fontmap.
- *
- * Since: 1.32.4
- **/
-guint
-pango_font_map_get_serial (PangoFontMap *fontmap)
-{
-  g_return_val_if_fail (PANGO_IS_FONT_MAP (fontmap), 0);
-
-  if (PANGO_FONT_MAP_GET_CLASS (fontmap)->get_serial)
-    return PANGO_FONT_MAP_GET_CLASS (fontmap)->get_serial (fontmap);
-  else
-    return 1;
-}
-
-/**
- * pango_font_map_changed:
- * @fontmap: a #PangoFontMap
- *
- * Forces a change in the context, which will cause any #PangoContext
- * using this fontmap to change.
- *
- * This function is only useful when implementing a new backend
- * for Pango, something applications won't do. Backends should
- * call this function if they have attached extra data to the context
- * and such data is changed.
- *
- * Since: 1.34
- **/
-void
-pango_font_map_changed (PangoFontMap *fontmap)
-{
-  g_return_if_fail (PANGO_IS_FONT_MAP (fontmap));
-
-  if (PANGO_FONT_MAP_GET_CLASS (fontmap)->changed)
-    PANGO_FONT_MAP_GET_CLASS (fontmap)->changed (fontmap);
-}
