@@ -24,14 +24,10 @@
  * 	http://en.wikipedia.org/wiki/Sample_Font_Displays_In_Other_Languages
  * 	Fetched on 2008-08-19
  *
- * WP
- *      Wikipedia, Article about the language
- *      Fetched on 2020-09-08
- *
  * GLASS
  * 	Kermit project's "I Can Eat Glass" list, also available in pango-view/
  * 	http://www.columbia.edu/kermit/utf8.html#glass
- * 	Fetched on 2008-08-19, updates on 2020-09-08
+ * 	Fetched on 2008-08-19
  *
  * KERMIT
  * 	Kermit project's Quick-Brown-Fox equivalents for other languages
@@ -53,10 +49,6 @@
  *
  * Needless to say, the list MUST be sorted on the language code.
  */
-/* Sacrificial define to make introspection happy. */
-#ifndef LANGUAGE
-#define LANGUAGE(x, y, z)
-#endif
 LANGUAGE(
 	 af	/* Afrikaans */,
 	 GLASS,
@@ -75,12 +67,6 @@ LANGUAGE(
 	 "Gvxam mincetu apocikvyeh: ñizol ce mamvj ka raq kuse bafkeh mew."
 	 /* Tale under the full moon: the chief chemamull and the clay old woman at the lake/sea. */
 	)
-LANGUAGE(
-	 as	/* Assamese */,
-	 WP,
-	 "সুপ্ৰভাত আপোনাক লগ পাই ভাল লাগিল।"
-	 /* Good morning, nice to meet you. */
-	)
 LANGUAGE(
 	 bar	/* Bavarian */,
 	 GLASS,
@@ -153,18 +139,6 @@ LANGUAGE(
 	 "Zwölf Boxkämpfer jagen Viktor quer über den großen Sylter Deich."
 	 /* Twelve boxing fighters drive Viktor over the great. */
 	)
-LANGUAGE(
-	 doi	/* Dogri */,
-	 WP,
-	 "अ'ऊं बमार आँ, मिगी डॉक्टर दी लोड़ ऐ।"
-	 /* I'm sick, I need a doctor. */
-	)
-LANGUAGE(
-         dv     /* Maldivian */,
-         WP,
-         "މާއްދާ 1 – ހުރިހާ އިންސާނުން ވެސް އުފަންވަނީ، ދަރަޖަ އާއި ޙައްޤު ތަކުގައި މިނިވަންކަމާއި ހަމަހަމަކަން ލިބިގެންވާ ބައެއްގެ ގޮތުގައެވެ."
-         /* Beginning of UDHR */
-        )
 LANGUAGE(
 	 el	/* Greek */,
 	 WP-SFD,
@@ -232,8 +206,8 @@ LANGUAGE(
 LANGUAGE(
 	 ga	/* Irish */,
 	 WP-PANG,
-	 "Chuaigh bé mhórshách le dlúthspád fíorfhinn trí hata mo dhea-phorcáin bhig."
-	 /* A maiden of large appetite with an intensely white, dense spade went through the hat of my good little porker. */
+	 "D'ḟuascail Íosa Úrṁac na hÓiġe Beannaiṫe pór Éaḃa agus Áḋaiṁ."
+	 /* Jesus, Son of the blessed Virgin, redeemed the seed of Eve and Adam. */
 	)
 LANGUAGE(
 	 gd	/* Scottish Gaelic */,
@@ -285,9 +259,9 @@ LANGUAGE(
 	)
 LANGUAGE(
 	 hr	/* Croatian */,
-	 MISC,
-	 "Deblji krojač: zgužvah smeđ filc u tanjušni džepić."
-	 /* A fatter taylor: I’ve crumpled a brown felt in a slim pocket. */
+	 GLASS,
+	 "Ja mogu jesti staklo i ne boli me."
+	 /* I can eat glass and it doesn't hurt me. */
 	)
 LANGUAGE(
 	 hu	/* Hungarian */,
@@ -341,12 +315,6 @@ LANGUAGE(
 	 "მინას ვჭამ და არა მტკივა."
 	 /* I can eat glass and it doesn't hurt me. */
 	)
-LANGUAGE(
-         km     /* Khmer */,
-         GLASS,
-         "ខ្ញុំអាចញុំកញ្ចក់បាន ដោយគ្មានបញ្ហារ"
-	 /* I can eat glass and it doesn't hurt me. */
-        )
 LANGUAGE(
 	 kn	/* Kannada */,
 	 GLASS,
@@ -359,12 +327,6 @@ LANGUAGE(
 	 "다람쥐 헌 쳇바퀴에 타고파"
 	 /* I Wanna ride on the chipmunk's old hamster wheel. */
 	)
-LANGUAGE(
-	 ks	/* Kashmiri */,
-	 WP,
-	 "ब’ छस न’ ज़ानान, तोहय हेकिवा दुबार’ वॅनिथ ।"
-	 /* I don't understand, Please say that again. */
-	)
 LANGUAGE(
 	 kw	/* Cornish */,
 	 GLASS,
@@ -376,12 +338,6 @@ LANGUAGE(
 	 WP-PANG,
 	 "Sic surgens, dux, zelotypos quam karus haberis"
 	)
-LANGUAGE(
-         lo     /* Lao */,
-         GLASS,
-         "ຂອ້ຍກິນແກ້ວໄດ້ໂດຍທີ່ມັນບໍ່ໄດ້ເຮັດໃຫ້ຂອ້ຍເຈັບ"
-	 /* I can eat glass and it doesn't hurt me. */
-        )
 LANGUAGE(
 	 lt	/* Lithuanian */,
 	 WP-PANG,
@@ -394,12 +350,6 @@ LANGUAGE(
 	 "Sarkanās jūrascūciņas peld pa jūru."
 	 /* Red seapigs swim in the sea. */
 	)
-LANGUAGE(
-	 mai	/* Maithili */,
-	 WP,
-	 "हम अहाँ के बिना कुछ नै छिय"
-	 /* I am nothing without you */
-	)
 LANGUAGE(
 	 map	/* Marquesan */,
 	 GLASS,
@@ -436,12 +386,6 @@ LANGUAGE(
 	 "Saya boleh makan kaca dan ia tidak mencederakan saya."
 	 /* I can eat glass and it doesn't hurt me. */
 	)
-LANGUAGE(
-         my     /* Burmese */,
-         WP,
-         "ဘာသာပြန်နှင့် စာပေပြုစုရေး ကော်မရှင်"
-         /* Literary and Translation Commission */
-        )
 LANGUAGE(
 	 nap	/* Neapolitan */,
 	 GLASS,
@@ -453,12 +397,6 @@ LANGUAGE(
 	 GSPECI,
 	 "Vår sære Zulu fra badeøya spilte jo whist og quickstep i min taxi."
 	)
-LANGUAGE(
-	 ne	/* Nepali */,
-	 WP,
-	 "के तपाईँ मसँग नाच्न चाहनुहुन्छ ?"
-	 /* Would you like to dance with me? */
-	)
 LANGUAGE(
 	 nl	/* Dutch */,
 	 WP-SFD,
@@ -520,9 +458,9 @@ LANGUAGE(
 	)
 LANGUAGE(
 	 pt-br	/* Brazilian Portuguese */,
-	 MISC, /* https://gitlab.gnome.org/GNOME/pango/-/issues/547 */
-         "Vejo galã sexy pôr quinze kiwis à força em baú achatado."
-         /* I see a sexy hunk forcefully stuff fifteen kiwis in a shallow trunk */
+	 WP-PANG,
+	 "À noite, vovô Kowalsky vê o ímã cair no pé do pingüim queixoso e vovó põe açúcar no chá de tâmaras do jabuti feliz."
+	 /* At night, grandpa Kowalsky sees the magnet falling in the complaining penguin's foot and grandma puts sugar in the happy tortoise's date tea.*/
 	)
 LANGUAGE(
 	 ro	/* Romanian */,
@@ -548,17 +486,6 @@ LANGUAGE(
 	 "Puotsu mangiari u vitru, nun mi fa mali."
 	 /* I can eat glass and it doesn't hurt me. */
 	)
-LANGUAGE(
-	 sd	/* Sindhi */,
-	 WP,
-	 "توهان ملي ڏاڍو سٺو لڳو"
-	 /* Pleased to meet you */
-	)
-LANGUAGE(
-         si     /* Sinhalese */,
-         WP,
-         "මනොපුබ්‌බඞ්‌ගමා ධම්‌මා, මනොසෙට්‌ඨා මනොමයා; මනසා චෙ පදුට්‌ඨෙන, භාසති වා කරොති වා; තතො නං දුක්‌ඛමන්‌වෙති, චක්‌කංව වහතො පදං."
-        )
 LANGUAGE(
 	 sk	/* Slovak */,
 	 KERMIT,
